@@ -1,6 +1,6 @@
 package My::Module::Meta;
 
-use 5.008001;
+use 5.008;
 
 use strict;
 use warnings;
@@ -17,47 +17,17 @@ sub new {
     return $self;
 }
 
-sub abstract {
-    return 'Play John Horton Conway\'s Game of Life, hopefully faster than a naive implementation.';
-}
-
-sub add_to_cleanup {
-    return [ qw{ cover_db xt/author/optionals } ];
-}
-
-sub author {
-    return 'Thomas R. Wyant, III F<wyant at cpan dot org>';
-}
-
 sub build_requires {
     return +{
-	'Test2::V0'			=> 0,
-	'Test2::Plugin::BailOnFail'	=> 0,
-	'Test2::Tools::LoadModule'	=> 0,
+	'Test::More'	=> 0.88,	# Because of done_testing().
     };
 }
 
-sub configure_requires {
-    return +{
-	'lib'	=> 0,
-	'strict'	=> 0,
-	'warnings'	=> 0,
-    };
-}
-
-sub dist_name {
-    return 'Game-Life-Faster';
-}
-
 sub distribution {
     my ( $self ) = @_;
     return $self->{distribution};
 }
 
-sub license {
-    return 'perl';
-}
-
 sub meta_merge {
     my ( undef, @extra ) = @_;
     return {
@@ -68,7 +38,6 @@ sub meta_merge {
 	resources	=> {
 	    bugtracker	=> {
 		web	=> 'https://rt.cpan.org/Public/Dist/Display.html?Name=Game-Life-Faster',
-		# web	=> 'https://github.com/trwyant/perl-Game-Life-Faster/issues',
 		mailto  => 'wyant@cpan.org',
 	    },
 	    license	=> 'http://dev.perl.org/licenses/',
@@ -82,20 +51,6 @@ sub meta_merge {
     };
 }
 
-sub module_name {
-    return 'Game::Life::Faster';
-}
-
-sub no_index {
-    return +{
-      directory => [
-                     'inc',
-                     't',
-                     'xt',
-                   ],
-    };
-}
-
 sub provides {
     -d 'lib'
 	or return;
@@ -114,7 +69,6 @@ sub requires {
     return +{
 	'Carp'		=> 0,
 	'List::Util'	=> 0,
-	constant	=> 0,
 	strict		=> 0,
 	warnings	=> 0,
 	@extra,
@@ -122,17 +76,9 @@ sub requires {
 }
 
 sub requires_perl {
-    return 5.008001;
+    return 5.008;
 }
 
-sub script_files {
-    return [
-    ];
-}
-
-sub version_from {
-    return 'lib/Game/Life/Faster.pm';
-}
 
 1;
 
@@ -167,19 +113,6 @@ This class supports the following public methods:
 
 This method instantiates the class.
 
-=head2 abstract
-
-This method returns the distribution's abstract.
-
-=head2 add_to_cleanup
-
-This method returns a reference to an array of files to be added to the
-cleanup.
-
-=head2 author
-
-This method returns the name of the distribution author
-
 =head2 build_requires
 
  use YAML;
@@ -190,20 +123,6 @@ modules required to build the C<Game::Life::Faster> package, suitable for
 use in a F<Build.PL> C<build_requires> key, or a F<Makefile.PL>
 C<< {META_MERGE}->{build_requires} >> or C<BUILD_REQUIRES> key.
 
-=head2 configure_requires
-
- use YAML;
- print Dump( $meta->configure_requires() );
-
-This method returns a reference to a hash describing the modules
-required to configure the package, suitable for use in a F<Build.PL>
-C<configure_requires> key, or a F<Makefile.PL>
-C<< {META_MERGE}->{configure_requires} >> or C<CONFIGURE_REQUIRES> key.
-
-=head2 dist_name
-
-This method returns the distribution name.
-
 =head2 distribution
 
  if ( $meta->distribution() ) {
@@ -215,32 +134,6 @@ This method returns the distribution name.
 This method returns the value of the environment variable
 C<MAKING_MODULE_DISTRIBUTION> at the time the object was instantiated.
 
-=head2 license
-
-This method returns the distribution's license.
-
-=head2 meta_merge
-
- use YAML;
- print Dump( $meta->meta_merge() );
-
-This method returns a reference to a hash describing the meta-data which
-has to be provided by making use of the builder's C<meta_merge>
-functionality. This includes the C<dynamic_config> and C<resources>
-data.
-
-Any arguments will be appended to the generated array.
-
-=head2 module_name
-
-This method returns the name of the module the distribution is based
-on.
-
-=head2 no_index
-
-This method returns the names of things which are not to be indexed
-by CPAN.
-
 =head2 provides
 
  use YAML;
@@ -270,20 +163,11 @@ may be added.
 
 This method returns the version of Perl required by the package.
 
-=head2 script_files
-
-This method returns a reference to an array containing the names of
-script files provided by this distribution. This array may be empty.
-
-=head2 version_from
-
-This method returns the name of the distribution file from which the
-distribution's version is to be derived.
-
 =head1 ATTRIBUTES
 
 This class has no public attributes.
 
+
 =head1 ENVIRONMENT
 
 =head2 MAKING_MODULE_DISTRIBUTION
@@ -292,12 +176,11 @@ This environment variable should be set to a true value if you are
 making a distribution. This ensures that no configuration-specific
 information makes it into F<META.yml>.
 
+
 =head1 SUPPORT
 
 Support is by the author. Please file bug reports at
-L<https://rt.cpan.org/Public/Dist/Display.html?Name=Game-Life-Faster>,
-L<https://github.com/trwyant/perl-Game-Life-Faster/issues>, or in
-electronic mail to the author.
+L<http://rt.cpan.org>, or in electronic mail to the author.
 
 =head1 AUTHOR
 
@@ -305,7 +188,7 @@ Thomas R. Wyant, III F<wyant at cpan dot org>
 
 =head1 COPYRIGHT AND LICENSE
 
-Copyright (C) 2019-2021 by Thomas R. Wyant, III
+Copyright (C) 2019 by Thomas R. Wyant, III
 
 This program is free software; you can redistribute it and/or modify it
 under the same terms as Perl 5.10.0. For more details, see the full text
