@@ -1,6 +1,6 @@
 package Game::Life::Faster;
 
-use 5.008001;
+use 5.008;
 
 use strict;
 use warnings;
@@ -8,7 +8,7 @@ use warnings;
 use Carp;
 use List::Util qw{ max min };
 
-our $VERSION = '0.006_01';
+our $VERSION = '0.001';
 
 use constant ARRAY_REF	=> ref [];
 
@@ -48,67 +48,39 @@ sub new {
     bless $self, ref $class || $class;
     $self->set_rules( $breed, $live );
 
-    $self->clear();
-
     return $self;
 }
 
 sub clear {
     my ( $self ) = @_;
     delete $self->{grid};
-    delete $self->{changed};
-    $self->{living_x} = [];
-    $self->{living_y} = [];
-    $self->{change_count} = 0;
     return $self;
 }
 
-sub get_active_grid_coord {
-    my ( $self ) = @_;
-    my ( $min_x, $max_x, $min_y, $max_y ) = ( $self->{size_x}, 0,
-	$self->{size_y}, 0 );
-    foreach my $ix ( keys %{ $self->{changed} } ) {
-	$min_x = $ix if $ix < $min_x;
-	$max_x = $ix if $ix > $max_x;
-	foreach my $iy ( keys %{ $self->{changed}{$ix} } ) {
-	    $min_y = $iy if $iy < $min_y;
-	    $max_y = $iy if $iy > $max_y;
-	}
-    }
-    $max_x < $min_x
-	and croak 'No active cells';
-    return [ $min_x, $max_x, $min_y, $max_y ];
-}
-
 sub get_breeding_rules {
     my ( $self ) = @_;
     return $self->get_rule( 'breed' );
 }
 
 sub get_grid {
-    my ( $self, $coord ) = @_;
-    $coord ||= $self->get_grid_coord();
-    $self->{grid}
-	or return [ ( [ ( 0 ) x ( $coord->[3] - $coord->[2] + 1 ) ] ) x
-	    ( $coord->[1] - $coord->[0] + 1 ) ];
-    my @rslt;
-    foreach my $x ( $coord->[0] .. $coord->[1] ) {
-	if ( $self->{grid}{$x} ) {
-	    push @rslt, [];
-	    foreach my $y ( $coord->[2] .. $coord->[3] ) {
-		push @{ $rslt[-1] }, $self->{grid}{$x}{$y} ?
-		    $self->{grid}{$x}{$y}[0] ? 1 : 0 : 0;
-	    }
-	} else {
-	    push @rslt, [ ( 0 ) x ( $coord->[3] - $coord->[2] + 1 ) ];
-	}
-    }
-    return \@rslt;
-}
-
-sub get_grid_coord {
     my ( $self ) = @_;
-    return [ 0, $self->{max_x}, 0, $self->{max_y} ];
+    if ( $self->{grid} ) {
+	my @rslt;
+	foreach my $x ( 0 .. $self->{max_x} ) {
+	    if ( $self->{grid}[$x] ) {
+		push @rslt, [];
+		foreach my $y ( 0 .. $self->{max_y} ) {
+		    push @{ $rslt[-1] }, $self->{grid}[$x][$y] ?
+			$self->{grid}[$x][$y][0] ? 1 : 0 : 0;
+		}
+	    } else {
+		push @rslt, [ ( 0 ) x $self->{size_y} ];
+	    }
+	}
+	return \@rslt;
+    } else {
+	return [ ( [ ( 0 ) x $self->{size_y} ] ) x $self->{size_x} ];
+    }
 }
 
 sub get_living_rules {
@@ -117,68 +89,54 @@ sub get_living_rules {
 }
 
 sub get_text_grid {
-    my ( $self, $living, $dead, $coord ) = @_;
+    my ( $self, $living, $dead ) = @_;
     $living	||= 'X';
     $dead	||= '.';
-    $coord ||= $self->get_grid_coord();
     my @rslt;
     if ( $self->{grid} ) {
-	foreach my $x ( $coord->[0] .. $coord->[1] ) {
-	    if ( $self->{grid}{$x} ) {
+	foreach my $x ( 0 .. $self->{max_x} ) {
+	    if ( $self->{grid}[$x] ) {
 		push @rslt, join '', map {
-		    ( $self->{grid}{$x}{$_} && $self->{grid}{$x}{$_}[0]) ?
+		    ( $self->{grid}[$x][$_] && $self->{grid}[$x][$_][0]) ?
 		    $living : $dead
-		} $coord->[2] .. $coord->[3];
+		} 0 .. $self->{max_y};
 	    } else {
-		push @rslt, $dead x ( $coord->[3] - $coord->[2] + 1 );
+		push @rslt, $dead x $self->{size_y};
 	    }
 	}
     } else {
-	@rslt = ( $dead x ( $coord->[3] - $coord->[2] + 1 ) ) x (
-	    $coord->[1] - $coord->[0] + 1 );
+	@rslt = ( $dead x $self->{size_y} ) x $self->{size_x};
     }
     return wantarray ? @rslt : join '', map { "$_\n" } @rslt;
 }
 
-sub get_active_text_grid {
-    my ( $self, $living, $dead ) = @_;
-    return $self->get_text_grid( $living, $dead,
-	$self->get_active_grid_coord() );
-}
-
-sub get_used_text_grid {
-    my ( $self, $living, $dead ) = @_;
-    return $self->get_text_grid( $living, $dead,
-	$self->get_used_grid_coord() );
-}
-
 sub get_used_grid {
     my ( $self ) = @_;
-    return $self->get_grid( $self->get_used_grid_coord() );
-}
-
-sub get_used_grid_coord {
-    my ( $self ) = @_;
-    my $min_x = $self->{size_x};
-    for ( $min_x = 0; $min_x < $self->{size_x}; $min_x++ ) {
-	$self->{living_x}[$min_x]
-	    or next;
-	my ( $max_x, $min_y, $max_y );
-	for ( $max_x = $self->{size_x}; $max_x >= $min_x; ) {
-	    $self->{living_x}[--$max_x]
-		and last;
+    $self->{grid}
+	or return;
+    my @rslt;
+    my $skip_x = 0;
+    foreach my $x ( 0 .. $self->{max_x} ) {
+	if ( $self->{grid}[$x] ) {
+	    push @rslt, ( undef ) x $skip_x;
+	    my $skip_y = $skip_x = 0;
+	    foreach my $y ( 0 .. $self->{max_y} ) {
+		if ( $self->{grid}[$x][$y] &&
+		    defined $self->{grid}[$x][$y][0] ) {
+		    @rslt > $x
+			or push @rslt, [];
+		    push @{ $rslt[-1] }, ( undef ) x $skip_y,
+			$self->{grid}[$x][$y][0];
+		    $skip_y = 0;
+		} else {
+		    $skip_y++;
+		}
+	    }
+	} else {
+	    $skip_x++;
 	}
-	for ( $min_y = 0; $min_y < $self->{size_y}; $min_y++ ) {
-	    $self->{living_y}[$min_y]
-		and last;
-	}
-	for ( $max_y = $self->{size_y}; $max_y >= $min_y; ) {
-	    $self->{living_y}[--$max_y]
-		and last;
-	}
-	return [ $min_x, $max_x, $min_y, $max_y ];
     }
-    croak 'No occupied cells';
+    return \@rslt;
 }
 
 sub place_points {
@@ -216,29 +174,32 @@ sub process {
     my ( $self, $steps ) = @_;
     $steps ||= 1;
 
+    my @toggle;
+
     foreach ( 1 .. $steps ) {
+	@toggle = ();
 
-	my $changed = delete $self->{changed};
-	$self->{change_count} = 0;
-
-	foreach my $x ( keys %{ $changed } ) {
-	    foreach my $y ( keys %{ $changed->{$x} } ) {
-		my $cell = $self->{grid}{$x}{$y};
+	foreach my $x ( keys %{ $self->{changed} } ) {
+	    foreach my $y ( keys %{ $self->{changed}{$x} } ) {
+		my $cell = $self->{grid}[$x][$y];
 		no warnings qw{ uninitialized };
 		if ( $cell->[0] ) {
-		    $self->{live}[ $changed->{$x}{$y} ]
-			or $self->unset_point( $x, $y );
+		    $self->{live}[ $cell->[1] ]
+			or push @toggle, [ $x, $y, 0 ];
 		} else {
-		    $self->{breed}[ $changed->{$x}{$y} ]
-			and $self->set_point( $x, $y );
+		    $self->{breed}[ $cell->[1] ]
+			and push @toggle, [ $x, $y, 1 ];
 		}
 	    }
 	}
 
-	$self->{change_count}
-	    or last;
+	delete $self->{changed};
+
+	foreach my $cell ( @toggle ) {
+	    $self->set_point_state( @{ $cell } );
+	}
     }
-    return $self->{change_count};
+    return scalar @toggle;
 }
 
 sub set_point {
@@ -248,7 +209,6 @@ sub set_point {
 
 sub set_point_state {
     my ( $self, $x, $y, $state ) = @_;
-
     defined $x
 	and defined $y
 	and $x =~ NON_NEGATIVE_INTEGER_RE
@@ -256,44 +216,39 @@ sub set_point_state {
 	or croak 'Coordinates must be non-negative integers';
     defined $state
 	or return $state;
-
-    if ( $x >= 0 && $x < $self->{size_x} &&
-	$y >= 0 && $y < $self->{size_y}
-    ) {
-	# We're on-grid.
-
-	# This autovivifies, but we're going to assign it anyway, so ...
-	my $prev_state = $self->{grid}{$x}{$y}[0] || 0;
-	$state = TOGGLE_STATE_REF eq ref $state ? 1 - $prev_state :
-	    $state ? 1 : 0;
-
-	$self->{grid}{$x}{$y}[0] = $state;
-	$self->{grid}{$x}{$y}[1] ||= 0;
-	my $delta = $state - $prev_state
-	    or return $state;
-	$self->{living_x}[$x] += $delta;
-	$self->{living_y}[$y] += $delta;
-
-	$self->{change_count}++;
-
-	foreach my $ix ( max( 0, $x - 1 ) .. min( $self->{max_x}, $x + 1 ) ) {
-	    foreach my $iy ( max( 0, $y - 1 ) .. min( $self->{max_y}, $y + 1 )
-	    ) {
-		$self->{changed}{$ix}{$iy} =
-		    $self->{grid}{$ix}{$iy}[1] += $delta;
-	    }
+    my $off_grid = ( $x < 0 || $x > $self->{max_x} ||
+	$y < 0 || $y > $self->{max_y} )
+	and $state
+	and croak 'Attempt to place living cell outside grid';
+    if ( TOGGLE_STATE_REF eq ref $state ) {
+	$state = 1;
+	$self->{grid}
+	    and $self->{grid}[$x]
+	    and $self->{grid}[$x][$y]
+	    and $state = ( ! $self->{grid}[$x][0] );
+    }
+    $state = $state ? 1 : 0;
+    my $prev_val = $off_grid ? 0 : $self->{grid}[$x][$y][0] ? 1 : 0;
+    unless ( $off_grid ) {
+	$self->{grid}[$x][$y][0] = $state;
+	$self->{grid}[$x][$y][1] ||= 0;
+    }
+    my $delta = $state - $prev_val
+	or return $state;
+    foreach my $ix ( max( 0, $x - 1 ) .. min( $self->{max_x}, $x + 1 ) ) {
+	foreach my $iy ( max( 0, $y - 1 ) .. min( $self->{max_y}, $y + 1 ) ) {
+	    $self->{grid}[$ix][$iy][1] += $delta;
+	    $self->{changed}{$ix}{$iy}++;
 	}
-
-	# A cell is not its own neighbor, but the above nested loops
-	# assumed that it was. We fix that here, rather than skip it
-	# inside the loops.
-	$self->{changed}{$x}{$y} =
-	    $self->{grid}{$x}{$y}[1] -= $delta;
-
-    } elsif ( $state ) {
-	croak 'Attempt to place living cell outside grid';
+    }
+    # A cell is not its own neighbor, but the above nested loops assumed
+    # that it was. We fix that here, rather than skip it inside the
+    # loops.
+    unless ( $off_grid ) {
+	$self->{grid}[$x][$y][1] -= $delta;
+	--$self->{changed}{$x}{$y}
+	    or delete $self->{changed}{$x}{$y};
     }
-
     return $state;
 }
 
@@ -313,14 +268,15 @@ sub set_point_state {
     sub set_rule {
 	my ( $self, $kind, $rule ) = @_;
 	$dflt{$kind}
-	    or croak "'$kind' is not a valid rule name";
+	    or croak "'$kind' is not a valid rule kind";
 	$rule ||= $dflt{$kind};
+	my $name = ucfirst $kind;
 	ARRAY_REF eq ref $rule
-	    or croak "\u$kind rule must be an array reference";
+	    or croak 'Breed rule must be an array reference';
 	$self->{$kind} = [];
 	foreach ( @{ $rule } ) {
 	    $_ =~ NON_NEGATIVE_INTEGER_RE
-		or croak "\u$kind rule must be a reference to an array of non-negative integers";
+		or croak "$name rule must be a reference to an array of non-negative integers";
 	    $self->{$kind}[$_] = 1;
 	}
 	return;
@@ -421,6 +377,7 @@ array. Order is not important.
 
 The default is C<[ 3 ]>.
 
+
 =item $live
 
 This specifies the number of neighbors a "living" cell needs to remain
@@ -437,36 +394,6 @@ invocant.
 
 This method is an extension to L<Game::Life|Game::Life>.
 
-=head2 get_active_grid_coord
-
- my $coord = $life->get_active_text_grid();
-
-This method returns the coordinates of the bounding rectangle for all
-active points in the grid -- that is, all whose value changed in the
-most-recent iteration (if any) or whose values were manually changed
-since the most-recent iteration. An exception is thrown if there are no
-active points.
-
-The return is a reference to an array containing the minimum and maximum
-X coordinate followed by the minimum and maximum Y coordinate; that is:
-
- [ $min_x, $max_x, $min_y, $max_y ]
-
-Note that these intervals are closed on both ends. To iterate over the
-active rows you would specify C<$min_x .. $max_x>.
-
-=head2 get_active_text_grid
-
- print $life->get_active_text_grid( $living, $dead )
-
-This convenience method returns the result of
-
- $life->get_text_grid( $living, $dead,
-     $life->get_active_grid_coord() )
-
-The arguments are the character to represent an occupied cell and the
-character to represent an empty cell.
-
 =head2 get_breeding_rules
 
  use Data::Dumper;
@@ -482,32 +409,13 @@ originally-specified order.
 
 =head2 get_grid
 
- my $grid = $life->get_grid( $coord )
+ use Data::Dumper;
+ print Dumper( $life->get_grid() );
 
-This method returns the grid as a reference to an array of array
-references. The argument is a reference to the minimum and maximum X and
-Y coordinates:
-
- [ $min_x, $max_x, $min_y, $max_y ]
-
-If the argument is omitted you get the entire grid.
-
-=head2 get_grid_coord
-
- my $coord = $life->get_grid_coord();
-
-This method returns the coordinates of the bounding rectangle for the
-entire grid.
-
-The return is a reference to an array containing the minimum and maximum
-X coordinate followed by the minimum and maximum Y coordinate. Assuming
-the size specified when the object was created was
-
- [ $size_x, $size_y ]
-
-the return will be
-
- [ 0, $size_x - 1, 0, $size_y - 1 ]
+This method returns the state of the grid, as a reference to an array of
+array references. The contents of the inner arrays represent the states
+of the cells, with a true value representing a "living" cell and a false
+value representing a "dead" cell.
 
 =head2 get_living_rules
 
@@ -565,35 +473,6 @@ As an incompatible change to the same-named method of
 L<Game::Life|Game::Life>, if called in scalar context this method
 returns a single string representing the entire grid.
 
-=head2 get_used_grid_coord
-
- my $coord = $life->get_used_grid_coord()
-
-This method returns the coordinates of the bounding rectangle for all
-occupied points in the grid.
-
-The return is a reference to an array containing the minimum and maximum
-X coordinate followed by the minimum and maximum Y coordinate; that is:
-
- [ $min_x, $max_x, $min_y, $max_y ]
-
-Note that these intervals are closed on both ends. To iterate over the
-active rows you would specify C<$min_x .. $max_x>.
-
-=head2 get_used_text_grid
-
- my ( $x, $y, $grid ) = $life->get_used_text_grid()
- print "${grid}at row $x column $y\n"
-
-This convenience method returns the living portion of the grid as
-text. Specifically, the returns are the number of the first row that
-contains a living cell, the number of the column that contains the first
-living cell, and the text grid with each line C<"\n">-terminated.
-
-If there are no living cells, nothing is returned.
-
-If called in scalar context you get the living portion of the grid.
-
 =head2 get_used_grid
 
  use Data::Dumper;
@@ -753,9 +632,7 @@ L<Game::Life|Game::Life>.
 =head1 SUPPORT
 
 Support is by the author. Please file bug reports at
-L<https://rt.cpan.org/Public/Dist/Display.html?Name=Game-Life-Faster>,
-L<https://github.com/trwyant/perl-Game-Life-Faster/issues>, or in
-electronic mail to the author.
+L<http://rt.cpan.org>, or in electronic mail to the author.
 
 =head1 AUTHOR
 
@@ -763,7 +640,7 @@ Thomas R. Wyant, III F<wyant at cpan dot org>
 
 =head1 COPYRIGHT AND LICENSE
 
-Copyright (C) 2019-2022 by Thomas R. Wyant, III
+Copyright (C) 2019 by Thomas R. Wyant, III
 
 This program is free software; you can redistribute it and/or modify it
 under the same terms as Perl 5.10.0. For more details, see the full text
